require "import"
import "android.content.Intent"
import "android.content.pm.ApplicationInfo"
import "android.content.Context"
import "android.content.pm.PackageManager"
import "android.widget.ArrayAdapter"
import "android.widget.GridView"
import "android.widget.LinearLayout"
import "android.widget.TextView"
import "android.widget.Toast"
import "android.widget.EditText"
import "android.os.Vibrator"
import "android.view.Gravity"
import "com.androlua.*"

local context = activity or service
local packageManager = context.getPackageManager()
local vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)

local selectedApp = nil

-- Function to open the selected application as normal
local function openSelectedApp()
    if selectedApp then
        local intent = packageManager.getLaunchIntentForPackage(selectedApp.packageName)
        if intent then
            context.startActivity(intent)
        else
Toast.makeText(context, "No launch intent found for selected app", Toast.LENGTH_SHORT).show()
        end
    else
Toast.makeText(context, "You must select an application", Toast.LENGTH_SHORT).show()
    end
end

-- Get list of installed apps
local installedApplications = packageManager.getInstalledApplications(PackageManager.GET_META_DATA)

-- Create a list of application names and a dictionary of applications with corresponding names
local appNames = {}
local installedAppInfos = {}
for i = 1, installedApplications.size() do
    local appInfo = installedApplications.get(i - 1)
    if (appInfo.flags & ApplicationInfo.FLAG_SYSTEM) == 0 then
        local appName = appInfo.loadLabel(packageManager)
        local packageName = appInfo.packageName
        local intent = packageManager.getLaunchIntentForPackage(packageName)
        if intent then
            installedAppInfos[appName] = appInfo
            appNames[#appNames + 1] = appName
        end
    end
end

-- Sort the list of application names in alphabetical order
table.sort(appNames)

-- Función para filtrar la lista de aplicaciones
local function filterApps(query)
    local filteredApps = {}
    for _, appName in ipairs(appNames) do
        if string.match(string.lower(appName), string.lower(query)) then
            table.insert(filteredApps, appName)
        end
    end
    return filteredApps
end

-- Crear una vista de cuadrícula utilizando AndroLua
local gridView = GridView(context)
gridView.setNumColumns(1)
gridView.setLayoutParams(LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.MATCH_PARENT))
local adapter = ArrayAdapter(context, android.R.layout.simple_list_item_1, appNames)
gridView.setAdapter(adapter)

-- Create an EditText for the search
local searchEditText = EditText(context)
searchEditText.setLayoutParams(LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT))
searchEditText.setHint("Look for...")

-- Create a variable to hold a reference to the current list of applications
local currentApps = appNames

-- Add event to detect changes in the EditText
searchEditText.addTextChangedListener({
    onTextChanged = function(s, start, before, count)
        local searchText = tostring(s)
        if #searchText > 0 then
            local filteredApps = filterApps(searchText)
            currentApps = filteredApps
            gridView.setAdapter(ArrayAdapter(context, android.R.layout.simple_list_item_1, filteredApps))
        else
            currentApps = appNames
            gridView.setAdapter(ArrayAdapter(context, android.R.layout.simple_list_item_1, appNames))
        end
    end
})

-- Create a vertical linear layout and add the title, the EditText and the grid view
local linearLayout = LinearLayout(context)
linearLayout.setOrientation(LinearLayout.VERTICAL)
linearLayout.setLayoutParams(LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.MATCH_PARENT))

local titleTextView = TextView(context)
titleTextView.setText("Select an app")
titleTextView.setTextSize(20)
titleTextView.setGravity(Gravity.CENTER)
titleTextView.setPadding(16, 16, 16, 16)

linearLayout.addView(titleTextView)
linearLayout.addView(searchEditText)
linearLayout.addView(gridView)

-- Create the dialog using LuaDialog and set the content as the linear layout
local dlg = LuaDialog(context)
dlg.setTitle("Select an application")
dlg.setView(linearLayout)

-- Manejar el evento de clic en la cuadrícula
gridView.setOnItemClickListener{
    onItemClick = function(parent, view, position, id)
        local appName = currentApps[position + 1]
        vibrator.vibrate(100)
        Toast.makeText(context, " " .. appName, Toast.LENGTH_SHORT).show()
        selectedApp = installedAppInfos[appName]
        openSelectedApp()
        dlg.dismiss()
    end
}

dlg.show()